__title__ = "cihai"
__package_name__ = "cihai"
__version__ = "0.17.1a0"
__description__ = "Library for CJK (chinese, japanese, korean) language data."
__author__ = "Tony Narlock"
__email__ = "tony@git-pull.com"
__github__ = "https://github.com/cihai/cihai"
__docs__ = "https://cihai.git-pull.com"
__tracker__ = "https://github.com/cihai/cihai/issues"
__pypi__ = "https://pypi.org/project/cihai/"
__license__ = "MIT"
__copyright__ = "Copyright 2013- cihai software foundation (Tony Narlock)"
